/*
 * button.cc
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <Arduino.h>
#include <button.h>


Button::Button( int pin )
	:
	_pin( pin ),
	_state( digitalRead( pin )? false : true ),
	_state_millis( ::millis() ),
	_state_duration( 0 )
{
}


void Button::add_event_at( int duration, int event_id )
{
	_press_events[ duration ] = event_id;
}


int Button::update()
{
	// reset event
	_event_id = 0;

	// get time
	unsigned long millis = ::millis();

	// get state and check for change
	bool state = digitalRead( _pin )? false : true;
	if( state != _state )
	{
		_state = state;
		_state_millis = millis;
		_state_duration = 0;
	}

	// if button has been pressed for any amount of time
	if( state && _state_millis < millis )
	{
		// calculate new duration for this state
		unsigned long duration = millis - _state_millis;

		// check through events to see if the button has been
		// pressed long enough to trigger one
		for( std::map< int, int >::iterator i = _press_events.begin();
			 i != _press_events.end(); i++ )
		{
			// if this event is in the future, we can stop looking
			if( duration < (unsigned long)i->first )
				break;

			// if this event happened since the previous update, we
			// can trigger it
			if( _state_duration < (unsigned long)i->first )
				_event_id = i->second;
		}

		// update the duration we've accounted for
		_state_duration = duration;
	}

	return _event_id;
}


int Button::get_triggered_event()
{
	return _event_id;
}
