/*
 * button.h
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _BUTTON_H_
#define _BUTTON_H_


#include <map>


class Button
{
public:

	/**
	 * Constructor.
	 *
	 * @param pin the pin to monitor
	 */
	Button( int pin );

	/**
	 * Add a duration at which the specified event id should be triggered.
	 *
	 * @oaram duration the time at which the event should be triggered
	 * @param event_id a unique identifier for the event (not 0)
	 */
	void add_event_at( int duration, int event_id );

	/**
	 * Update internal state
	 */
	int update();

	/**
	 * Retrieve the event id of the event that was triggered at the last
	 * update, if there was one.
	 *
	 * @return event id, or 0
	 */
	int get_triggered_event();

protected:

	/** pin */
	int _pin;

	/** current state */
	bool _state;

	/** time when current state began */
	unsigned long _state_millis;

	/** milliseconds that have been accountred for (in terms of
	 * triggering events) since current state began */
	unsigned long _state_duration;

	/** triggered event */
	int _event_id;

	/** press events durations -> event ids */
	std::map< int, int > _press_events;

};


#endif //_BUTTON_H_
