/*
 * mode_switcher.cc
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "mode_switcher.h"
#include "test_mode.h"
#include "analogue_clock_mode.h"


ModeSwitcher::ModeSwitcher()
{
	static TestMode test_mode;
	static AnalogueClockMode analogue_clock_mode;

	_modes.push_back( &test_mode );
	_modes.push_back( &analogue_clock_mode );
}


void ModeSwitcher::short_press()
{
	_modes[ _mode ]->next_flavour();
}


void ModeSwitcher::long_press()
{
	if( ++_mode >= (signed)_modes.size() )
		_mode = 0;
	_modes[ _mode ]->activate();
}


void ModeSwitcher::activate()
{
	_mode = 0;
	_modes[ _mode ]->activate();
}


Drawer &ModeSwitcher::get_drawer()
{
	return *_modes[ _mode ];
}
