/*
 * mode_switcher.h
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _MODE_SWITCHER_H_
#define _MODE_SWITCHER_H_


#include "major_mode.h"
#include <vector>


class MinorMode;
class Drawer;


class ModeSwitcher
    :
    public MajorMode
{
public:

	ModeSwitcher();

	/**
	 * Called to inform the major mode that there has been a short press.
	 */
	void short_press();

	/**
	 * Called to inform the major mode that there has been a long press.
	 */
	void long_press();

	/**
	 * Called when this major mode is becoming active
	 */
	void activate();

	/**
	 * Retrieve a drawer.
	 *
	 * @return drawer
	 */
	Drawer &get_drawer();

private:

	/* minor modes that this mode switcher switches */
	std::vector< MinorMode * > _modes;

	/* current mode */
	int _mode;

};


#endif //_MODE_SWITCHER_H_
