/*
 * time.h
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _TIME_H_
#define _TIME_H_


class Time
{
public:

	/**
	 * Get singleton instance.
	 *
	 * @return instance
	 */
	static Time &get_instance();

	/**
	 * Get hours.
	 *
	 * @return hours, 0 to 23
	 */
	int get_hours();

	/**
	 * Get minutes.
	 *
	 * @return minutes, 0 to 59
	 */
	int get_minutes();

	/**
	 * Get seconds.
	 *
	 * @return seconds, 0 to 59
	 */
	int get_seconds();

	/**
	 * Update the time.
	 */
	void update();

protected:

	Time();

	/** hours */
	int _hours;

	/** minutes */
	int _minutes;

	/** seconds */
	int _seconds;

private:

	/** milliseconds at last update */
	unsigned long _last_millis;

	/** milliseconds carries over from last update */
	unsigned long _carry;

};


#endif _TIME_H_
